#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <imsls.h>
int main(){
   int n_obs       = 12;
   int n_locations = 1;
   int n_factors   = 3;
   int n_levels[4] ={3, 2, 2, 1}; 
   int page_width = 132;
   /*  model information */
   int model[]={
      1, 1, 1, 1,
         1, 1, 1, 2,
         1, 1, 2, 1,
         1, 1, 2, 2,
         1, 2, 1, 1,
         1, 2, 1, 2,
         1, 2, 2, 1,
         1, 2, 2, 2,
         1, 3, 1, 1,
         1, 3, 1, 2,
         1, 3, 2, 1,
         1, 3, 2, 2
   };
   /* response data */
   float y[] ={
      4.42725419998168950,  
         2.12795543670654300, 
         2.55254390835762020, 
         1.21479606628417970,
         2.47588264942169190, 
         5.01306104660034180, 
         4.73502767086029050, 
         4.58392113447189330, 
         5.01421167794615030, 
         4.11972457170486450, 
         6.51671624183654790, 
         4.73365202546119690
   };

   int model_order;
   int i, j, k, l, m, n_missing, i2, j2;
   int n_factor_levels=0, n_treatments=1;
   int n_two_way_means=0, n_two_way_std_err=0;
   int n_two_way_interactions=0;
   int n_subscripts, n_anova_table=2;
   float cv, grand_mean;
   float *anova_table;
   float *two_way_means, *two_way_std_err;
   float *treatment_means, *treatment_std_err;
   float *factor_means;
   float *factor_std_err; 
   float aNaN = imsls_f_machine(6);
   char  **anova_row_labels;
   char *col_labels[] = {" ", "\nID", "\nDF", "\nSSQ  ", 
      "Mean  \nsquares", "\nF-Test", "\np-Value"};
   /* 
   * Compute the length of some of the output arrays.
   */
   model_order = n_factors-1;
   for (i=0; i < n_factors; i++){
      n_factor_levels = n_factor_levels + n_levels[i];
      n_treatments    = n_treatments*n_levels[i];
      for (j=i+1; j < n_factors; j++){
         n_two_way_interactions++;
      }
   }
   n_two_way_std_err = n_two_way_interactions;
   for (i=0; i < n_factors-1; i++){
      for (j=i+1; j < n_factors; j++){
         n_two_way_means = n_two_way_means + n_levels[i]*n_levels[j];
      }
   } 
   n_subscripts = n_factors;
   n_anova_table = 2;
   for (i=1; i <= model_order; i++){
      n_anova_table += 
         (int)imsls_f_binomial_coefficient(n_subscripts, i);
   }   

   /* Set observation 9 to missing. */
   y[8] = aNaN;
   anova_table = imsls_f_crd_factorial(n_obs, n_locations, n_factors,
      n_levels, model, y,
      IMSLS_N_MISSING, &n_missing, 
      IMSLS_CV, &cv, 
      IMSLS_GRAND_MEAN, &grand_mean,
      IMSLS_FACTOR_MEANS, &factor_means,
      IMSLS_FACTOR_STD_ERRORS,   &factor_std_err,
      IMSLS_TWO_WAY_MEANS, &two_way_means,   
      IMSLS_TWO_WAY_STD_ERRORS,    &two_way_std_err,
      IMSLS_TREATMENT_MEANS, &treatment_means, 
      IMSLS_TREATMENT_STD_ERROR, &treatment_std_err,
      IMSLS_ANOVA_ROW_LABELS, &anova_row_labels,
      0) ;
   /* Output results. */

   imsls_page(IMSLS_SET_PAGE_WIDTH, &page_width);
   /* Print ANOVA table. */
   imsls_f_write_matrix("   *** ANALYSIS OF VARIANCE TABLE ***", 
      n_anova_table, 6, anova_table, 
      IMSLS_WRITE_FORMAT, "%3.0f%3.0f%8.3f%8.3f%8.3f%8.3f",
      IMSLS_ROW_LABELS, anova_row_labels,
      IMSLS_COL_LABELS, col_labels,
      0);
   printf("\n\nNumber of Missing Values Estimated: %d", n_missing);
   printf("\nGrand Mean:                       %7.3f", grand_mean);
   printf("\nCoefficient of Variation:         %7.3f", cv);

   m=0;
   /* Print Factor Means. */
   printf("\n\nFactor Means\n");
   for(i=0; i < n_factors; i++){
      printf("  Factor %d: ", i+1);
      for(j=0; j < n_levels[i]; j++){
         printf("  %f ", factor_means[m]);
         m++;
      }
      k = (int)factor_std_err[2*i+1];
      printf("\n              std. err.(df):        %f(%d) \n", 
         factor_std_err[2*i], k);
   }

   /* Print Two-Way Means. */
   printf("\n\nTwo-Way Means");
   m = 0;
   l=0;
   for(i=0; i < n_factors-1; i++){
      for(j=i+1; j < n_factors; j++){
         printf("\n  Factor %d by Factor %d: \n", i+1, j+1);
         for(i2=0; i2 < n_levels[i]; i2++){
            for(j2=0; j2 < n_levels[j]; j2++){
               printf("  %f ",two_way_means[m]);
               m++;
            }
            printf("\n");
         }
         k = (int)two_way_std_err[l+1];
         printf("  std. err.(df): = %f(%d) \n", two_way_std_err[l], k);
         l+=2;
      }
   }

   /* Print Treatment Means. */
   printf("\n\nTreatment Means\n");
   m = 0;
   for(i=0; i < n_levels[0]; i++){
      for(j=0; j < n_levels[1]; j++){
         for(k=0; k < n_levels[2]; k++){
            printf("  Treatment[%d][%d][%d] Mean: %f \n",
               i+1, j+1, k+1, treatment_means[m]);
            m++;
         }
      }
   }
   k = (int)treatment_std_err[1];
   printf("\n  Treatment Std. Err (df) %f(%d) \n", 
      treatment_std_err[0], k);
}

